/* Arithmetic stuff for calculation the buehlmann model */
/* $Id: buehlmann.c,v 1.7 2004-02-02 22:04:51 mattzz Exp $ */


#include <math.h>
#include <string.h>
#include "buehlmann.h"

float buehlmann_compart_model[16][8] = {
  /*
   * t1/2   a       b   p.a.tol p_te  te
   *
   */
  {4.0,   1.2599, 0.5050, 0.0, 0.0, 0.0, 0.0, 0.0},
  {8.0,   1.0000, 0.6514, 0.0, 0.0, 0.0, 0.0, 0.0},
  {12.5,  0.8618, 0.7222, 0.0, 0.0, 0.0, 0.0, 0.0},
  {18.5,  0.7562, 0.7825, 0.0, 0.0, 0.0, 0.0, 0.0},
  {27.0,  0.6667, 0.8126, 0.0, 0.0, 0.0, 0.0, 0.0},
  {38.3,  0.5933, 0.8434, 0.0, 0.0, 0.0, 0.0, 0.0},
  {54.3,  0.5282, 0.8693, 0.0, 0.0, 0.0, 0.0, 0.0},
  {77.0,  0.4710, 0.8910, 0.0, 0.0, 0.0, 0.0, 0.0},
  {109.0, 0.4187, 0.9092, 0.0, 0.0, 0.0, 0.0, 0.0},
  {146.0, 0.3798, 0.9222, 0.0, 0.0, 0.0, 0.0, 0.0},
  {187.0, 0.3497, 0.9319, 0.0, 0.0, 0.0, 0.0, 0.0},
  {239.0, 0.3223, 0.9403, 0.0, 0.0, 0.0, 0.0, 0.0},
  {305.0, 0.2971, 0.9477, 0.0, 0.0, 0.0, 0.0, 0.0},
  {390.0, 0.2737, 0.9544, 0.0, 0.0, 0.0, 0.0, 0.0},
  {498.0, 0.2523, 0.9602, 0.0, 0.0, 0.0, 0.0, 0.0},
  {635.0, 0.2327, 0.9653, 0.0, 0.0, 0.0, 0.0, 0.0}};


void calculate_pressure(dive_data *dive){ 

  dive->p_amb = dive->p_amb_air +  (dive->depth/10);

  dive->pp_n2 = ((100.0 - (dive->perc_o2 + dive->perc_he)) /
		 100.0) * (dive->p_amb - P_STEAM);

  dive->pp_o2 = (dive->perc_o2 / 100) * (dive->p_amb);
}

void init_dive_data(dive_data *dive){
  int i;

  /* my_dive members init */
  dive->deco         = 0;
  dive->deltamode    = 0;
  dive->depth        = 0.0;
  dive->depth_old    = 0.0;
  dive->time         = 0.0;
  dive->p_amb        = 0.0;
  dive->pp_n2        = 0.0; 
  dive->pp_o2        = 0.0;
  dive->pp_he        = 0.0;
  dive->p_amb_air    = 1.001;
  dive->perc_o2      = 21.0;
  dive->perc_he      = 0.0;
  dive->delta_time   = 60.0;
  dive->te_min       = 999.0;
  dive->depth_max    = 0.0;
  dive->depth_tol_min = 0.0;
  dive->depth_samples[0] = 0.0; /* dive starts always with depth=0.0! */
  dive->num_samples = 1;        /* for that reason we have on offset of 1 */

  memcpy(dive->compart, buehlmann_compart_model,
	 sizeof(buehlmann_compart_model));

  calculate_pressure(dive);

  /* inert gas pressure in balance, t0=te in all tissues */
  for (i=0; i<16; i++){
    dive->compart[i][C_PIG] = dive->pp_n2;
  }
}


void calculate_tissues(dive_data *dive){

  int i;
  float ptol, x, ndl;
  float ppn2, t12, a, b;
 
  calculate_pressure(dive);

  dive->depth_old = dive->depth;

  if(!dive->deco){
    dive->te_min = 999.0;
  }
  dive->pp_amb_tol_max = 0.0;
  dive->def_cmpt = 0;

  /* store max. depth */
  if (dive->depth > dive->depth_max){
    dive->depth_max = dive->depth;
  }

 
  for (i=0; i<16; i++){
    /* tissue saturation */
    // pt. i.g. (tE) = pt. i.g. (t0) + [pI i.g. - pt. i.g. (t0)]*[1 - 2^(-tE / t1/2)]
	
    dive->compart[i][C_PIG] = dive->compart[i][C_PIG] + 
      (dive->pp_n2 - dive->compart[i][C_PIG]) * 
      (1-pow(2,(-(dive->delta_time/60)/dive->compart[i][C_T1_2])));

    /* tolerated ambient pressure */
    // pamb. tol. = (pt. i.g. - a) * b
    dive->compart[i][C_PAMBTOL] = (dive->compart[i][C_PIG] -
				   dive->compart[i][C_A]) * dive->compart[i][C_B];

    if (dive->compart[i][C_PAMBTOL] > dive->pp_amb_tol_max) {
      dive->pp_amb_tol_max = dive->compart[i][C_PAMBTOL];
      dive->def_cmpt = i + 1;
    }

    /* check for deco transition: */
    /* are we going to deco? */
    /* is the tolerated ambient pressure greater than air pressure? */
    if (!dive->deco && (dive->pp_amb_tol_max > dive->p_amb_air)){
      dive->deco = 1;
      dive->te_min = 0; /* ndl is 0 */
    }
    /* are we leaving deco ?*/
    if (dive->deco && (dive->pp_amb_tol_max < dive->p_amb_air)){
      dive->deco = 0;
      dive->te_min = 999;
    }

    /* tolerated inert gas pressure in tissue */
    /* pt. tol. i.g. = (pamb / b) + a         */
    dive->compart[i][C_PIG_TOL]= (dive->p_amb / dive->compart[i][C_B]) + 
      dive->compart[i][C_A]; 

    /* NDL */
    if (!dive->deco){

      t12   = dive->compart[i][C_T1_2]; 
      a     = dive->compart[i][C_A];
      b     = dive->compart[i][C_B];
      ptol  = (dive->p_amb_air/b) + a;
      ppn2  = dive->pp_n2;


      x = ((ppn2 - ptol) / (ppn2 - dive->compart[i][C_PIG]));

      if (x > 0){

	ndl =  (-t12 * log(x)  / log(2));
	if (ndl>=0){
	  dive->compart[i][C_TE] = ndl;
	  /* save minimum */
	  if (ndl < dive->te_min){
	    dive->te_min = ndl;
	  } 
	} else {
	  // if !ndl>=0
//	  dive->deco = 1;
//	  dive->compart[i][C_TE] = 0;
//	  dive->te_min = 0;
	}
      }
    }
  }

  /* minimum tolerated depth */
  dive->depth_tol_min = (dive->pp_amb_tol_max - dive->p_amb_air ) * 10.0;

  /* store dive depth and depth_tol_min (deco ceiling) in sample array */
  dive->depth_samples[dive->num_samples]=dive->depth;
  dive->depth_tol_min_samples[dive->num_samples]=dive->depth_tol_min;
  dive->num_samples++;



  /* increase delta_t */
  dive->time = dive->time + dive->delta_time;

}
 

void calculate_blend(float eanx_from,
		     float start_pressure,
		     float eanx_to,
		     float end_pressure,
		     float *oxygen,
		     float *topping){
  
  *oxygen = (((eanx_to/100) * end_pressure) - ((eanx_from/100) * start_pressure) - 0.21*(end_pressure - start_pressure)) / 0.79;
  *topping = end_pressure - (start_pressure + *oxygen);
}

void calculate_best_mix(float ppo2_max, 
			float depth,
			float *eanx_best){

  *eanx_best = ppo2_max / (depth/10 + 1) * 100;
}

void calculate_ead(float eanx,
		   float depth,
		   float *ead){

  *ead = ((1-(eanx/100))*(depth+10)/0.79 ) - 10;
}

void calculate_mod(float eanx,
		   float ppo2_max,
		   float *mod){

  *mod = ((1000 * ppo2_max) / eanx) - 10;
}
